function [I] = IHSIC_estimation(Y,ds,co)
%Estimates mutual information (I) using the HSIC (Hilbert-Schmidt independence criterion) method. 
%
%We use the naming convention 'I<name>_estimation' to ease embedding new mutual information estimation methods.
%
%INPUT:
%   Y: Y(:,t) is the t^th sample.
%  ds: subspace dimensions.
%  co: mutual information estimator object.
%
%REFERENCE: 
%   Arthur Gretton, Olivier Bousquet, Alexander Smola and Bernhard Scholkopf. Measuring Statistical Dependence with Hilbert-Schmidt Norms. International Conference on Algorithmic Learnng Theory (ALT), 63-78, 2005.
%
%Copyright (C) 2012 Zoltan Szabo ("http://nipg.inf.elte.hu/szzoli", "szzoli (at) cs (dot) elte (dot) hu")
%
%This file is part of the ITE (Information Theoretical Estimators) Matlab/Octave toolbox.
%
%ITE is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with ITE. If not, see <http://www.gnu.org/licenses/>.

%co.mult:OK.

%verification:
    if sum(ds) ~= size(Y,1);
        error('The subspace dimensions are not compatible with Y.');
    end

%initialization:
    num_of_samples = size(Y,2);
    num_of_comps = length(ds);
    cum_ds = cumsum([1;ds(1:end-1)]);%1,d_1+1,d_1+d_2+1,...,d_1+...+d_{M-1}+1 = starting indices of the subspaces (M=number of subspaces).
    Gs = {};   
    
%Gs:   
   for k = 1 : num_of_comps
       ind = [cum_ds(k):cum_ds(k)+ds(k)-1];
       %Cholesky decomposition:
            [G,p] = chol_gauss(Y(ind,:),co.sigma,num_of_samples*co.eta);
       [temp,p] = sort(p); %p:=inverse of p
       Gs{k} = G(p,:);
   end

I = hsicChol(Gs,num_of_samples,num_of_comps);
